(ns com.aurellem.title
  (:use (com.aurellem gb-driver vbm)))

(defn first-difference [base alt summary root]
  (loop [branch-point root
         actions []]
    (let [base-branch (step branch-point base)
          base-val (summary base-branch)
          alt-branch (step branch-point alt)
          alt-val (summary alt-branch)]
      (if (not= base-val alt-val)
        [(conj actions alt) alt-branch]
        (recur base-branch (conj actions base))))))

(defn advance
  ([base alt summary [commands state]]
     (let [[c s] (first-difference base alt summary state)]
       [(concat commands c) s]))
  ([base alt [commands state]]
     (advance base alt AF [commands state]))
  ([alt [commands state]]
     (advance [] alt [commands state])))

(def scroll-text (partial advance [:b] [:a :b]))

(defn start [] [[] (root)])

(defn-memo title [start]
  (->> start
       (advance [] [:a])
       (advance [] [:start])
       (advance [] [:a])
       (advance [] [:start])))

(defn-memo oak [start]
  (->> (title)
       scroll-text
       scroll-text
       scroll-text
       scroll-text
       scroll-text
       scroll-text
       scroll-text
       scroll-text
       scroll-text
       scroll-text
       scroll-text
       scroll-text
       scroll-text
       (advance [] [:a])))

(defn-memo name-entry []
  (->> (oak)
       (advance [] [:r] DE)
       (play-moves
        [[]
         [:r] [] [:r] [] [:r] [] [:r] []
         [:r] [] [:r] [] [:r] [] [:d] [:a]  
         [:l] [] [:l] [] [:l] [] [:l] []
         [:l] [] [:l] [:a] [] [:r] [:a]
         [:r] [] [:r] [] [:r] [] [:r] []
         [:r] [] [:d] [] [:d] [] [:d] [:a]
         ])))
          
(defn-memo rival-name-entry []
  (->> (name-entry)
       scroll-text
       scroll-text
       scroll-text
       scroll-text
       scroll-text
       (advance [] [:d])
       (advance [] [:d])
       (advance [] [:a])))

(defn-memo finish-title []
  (->> (rival-name-entry)
       scroll-text
       scroll-text
       scroll-text
       scroll-text
       scroll-text
       scroll-text
       scroll-text))

(defn-memo intro []
  (-> (start) title oak name-entry rival-name-entry finish-title))


;; TODO might be able to glue these together more elegantly with monads

